-- Mergesort function used to insert data
-- Merge operation for mergesorting the @unit-sized
-- components of an array stored in a varbinary(8000)
-- removes duplicates if @distinct = 1
create function VarBinaryMerge(
  @v1 varbinary(4000),
  @v2 varbinary(4000),
  @unit smallint,
  @distinct bit
) returns varbinary(8000) as begin

  declare @v varbinary(8000)
  declare @vlast varbinary(8000)
  declare @p1 int, @p2 int
  set @p1 = 1 set @p2 = 1

  if substring(@v1,@p1,@unit) < substring(@v2,@p2,@unit) begin
    set @v = substring(@v1,@p1,@unit)
    set @p1 = @p1 + @unit
  end else begin
    set @v = substring(@v2,@p2,@unit)
    set @p2 = @p2 + @unit
  end
  set @vlast = @v

  declare @L1 int, @L2 int
  set @L1 = datalength(@v1)
  set @L2 = datalength(@v2)

  while @p1 <= @L1 and @p2 <= @L2 begin
    if substring(@v1,@p1,@unit) < substring(@v2,@p2,@unit) begin
      if @distinct = 0 or substring(@v1,@p1,@unit) > @vlast begin
        set @vlast = substring(@v1,@p1,@unit)
        set @v = @v + @vlast
      end
      set @p1 = @p1 + @unit
    end else begin
      if @distinct = 0 or substring(@v2,@p2,@unit) > @vlast begin
        set @vlast = substring(@v2,@p2,@unit)
        set @v = @v + @vlast
      end
      set @p2 = @p2 + @unit
    end
  end
  if @distinct = 0
    set @v = @v + substring(@v1,@p1,8000) + substring(@v2,@p2,8000)
  else begin
    while @p1 <= @L1 begin
      if substring(@v1,@p1,@unit) > @vlast begin
        set @vlast = substring(@v1,@p1,@unit)
        set @v = @v + @vlast
      end
      set @p1 = @p1 + @unit
    end
    while @p2 <= @L2 begin
      if substring(@v2,@p2,@unit) > @vlast begin
        set @vlast = substring(@v2,@p2,@unit)
        set @v = @v + @vlast
      end
      set @p2 = @p2 + @unit
    end
  end
return @v
end
go

-- Classic Mergesort
create function VarBinarySort(
  @v        varbinary(8000),
  @unit     int,
  @distinct bit
) returns varbinary(8000) as begin

  if datalength(@v) = @unit return @v
  declare @mid int
  set @mid = (datalength(@v)/@unit)/2*@unit
  return dbo.VarBinaryMerge(
           substring(@v,1,@mid),
           substring(@v,@mid+1,8000),
           @unit,@distinct
  )
end
go
/* -- examples
select
  cast(dbo.VarBinarySort(cast('12,33,22,11,12,88,' as varbinary(8000)),3,0) as varchar(8000))
select cast(dbo.VarBinarySort(cast('12,33,22,11,12,88,' as varbinary(8000)),3,1) as varchar(8000))


















select dbo.VarBinarySort(0x1234567822223399, 1, 1)
select dbo.VarBinarySort(0x1234567822223399, 1, 0)
select dbo.VarBinarySort(0x1234567822223399, 2, 1)
select dbo.VarBinarySort(0x000000000000, 2, 1)
select dbo.VarBinarySort(0x11222233, 1, 1)
select dbo.VarBinarySort(0x11222233, 1, 0)
*/
go

CREATE TABLE Courses
(
   courseid    INT             NOT NULL,
   coursename  VARCHAR(20)     NOT NULL,
   materialIDs varbinary(800) NOT NULL DEFAULT 0x,
   -- [materialIDs] is essentially a subtable implemented as a
   --   packed array of smallints, ordered and made distinct when inserted
   PRIMARY KEY(materialIDs, courseid)
)

-- Currently not used 
CREATE TABLE Materials
(
   materialid   smallint    NOT NULL PRIMARY KEY,
   materialname VARCHAR(20) NOT NULL,
   /* other columns */
)
go

SET NOCOUNT ON

DECLARE
  @nummaterials int,
  @materialid int,
  @numcourses int,
  @courseid int,
  @i int,
  @rnd int

set @nummaterials = 25
-- More overlap in material lists if this value is small

begin tran
  set @materialid = 1
  while @materialid <= @nummaterials
  begin
    insert into Materials
      values (@materialid, 'Material ' + ltrim(@materialid))
    set @materialid = @materialid + 1
  end
commit


set @numcourses = 50000

begin tran
  declare @m varbinary(8000)
  set @courseid = 1
  while @courseid <= @numcourses
  begin
    -- if @courseid % 1000 = 0 select @courseid
    set @rnd = abs(rand()+rand()+rand())*15+1
    set @m = 0x
    -- initially create materials list array without regard to
    -- duplicates or ordering
    while @rnd > 0 begin
      set @rnd = @rnd - 1
      set @m = @m
        + cast(cast(abs(rand()-1e-10)*@nummaterials+1
                    as smallint) as binary(2))
    end
    insert into Courses values (
      @courseid,
      'Course ' + ltrim(@courseid),
      dbo.VarBinarySort(@m,2,1)  -- remove duplicates and sort
    )
    set @courseid = @courseid + 1
  end
commit
go

-- select * from Courses

declare @d datetime
set @d = getdate()

select
-- retrieve every course id
  courseid, (
-- along with the one that is smallest-numbered

    select min(courseid) from
    Courses C2
-- that uses exactly the same set of materials
    where C1.materialIDs = C2.materialIDs
  ) as mincourseid
from Courses C1
order by courseid

select datediff(ms, @d, getdate())
go

drop table Courses, Materials
drop function VarBinaryMerge, VarBinarySort
