SET NOCOUNT ON
-- one table to hold any number of matrices of different sizes.
CREATE TABLE Matrices (
    MatrixNo int,
        row int,
        col int,
        val int
)
go
CREATE FUNCTION MatrixProduct (
  @matANo int,
  @matBNo int
) returns table as return
  SELECT A.row, B.col, SUM(A.val*B.val) AS val
  FROM Matrices A JOIN Matrices B
  ON A.MatrixNo = @matANo
  AND B.MatrixNo = @matBNo
  AND A.col = B.row
  GROUP BY A.row, B.col
GO

INSERT INTO Matrices VALUES (1,1,1,1)
INSERT INTO Matrices VALUES (1,1,2,2)
INSERT INTO Matrices VALUES (1,1,3,0)
INSERT INTO Matrices VALUES (1,2,1,2)
INSERT INTO Matrices VALUES (1,2,2,1)
INSERT INTO Matrices VALUES (1,2,3,1)
INSERT INTO Matrices VALUES (1,3,1,0)
INSERT INTO Matrices VALUES (1,3,2,1)
INSERT INTO Matrices VALUES (1,3,3,1)

INSERT INTO Matrices VALUES (2,1,1,2)
INSERT INTO Matrices VALUES (2,1,2,3)
INSERT INTO Matrices VALUES (2,1,3,1)
INSERT INTO Matrices VALUES (2,2,1,4)
INSERT INTO Matrices VALUES (2,2,2,0)
INSERT INTO Matrices VALUES (2,2,3,0)
INSERT INTO Matrices VALUES (2,3,1,0)
INSERT INTO Matrices VALUES (2,3,2,1)
INSERT INTO Matrices VALUES (2,3,3,1)

CREATE UNIQUE CLUSTERED INDEX Matrices_mrc ON Matrices(MatrixNo, row, col)
GO

create procedure ShowMatrix (
  @matrixNo int
) as 
PRINT 'Matrix '+ rtrim(@matrixNo)

SELECT CAST('R'+convert(varchar,row) AS VARCHAR(6)) AS [\],
       MAX(CASE WHEN col=1 THEN val END) AS [C1],
       MAX(CASE WHEN col=2 THEN val END) AS [C2],
       MAX(CASE WHEN col=3 THEN val END) AS [C3]
FROM Matrices
WHERE MatrixNo=@matrixNo
GROUP BY row
GO

exec ShowMatrix 1
GO
exec ShowMatrix 2
GO
insert into Matrices(MatrixNo, row, col, val)
select 3, row, col, val
from MatrixProduct(1,2)

exec ShowMatrix 3

GO

DROP TABLE Matrices
drop function MatrixProduct
drop procedure ShowMatrix 